﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DesignPattern.FactoryMethodPattern.FactoryMethod;

namespace DesignPattern.FactoryMethodPattern.BatchFactory
{
    /// <summary>
    /// 批量加工接口
    /// </summary>
    public interface IBatchFactory
    {
        VideoPlayerList CreateVideoPlayers(int quantity);
    }
    /// <summary>
    /// 批量加工的抽象基类
    /// </summary>
    public abstract class BatchVideoPlayerFactory<T> : IBatchFactory 
        where T : VideoPlayer, new()
    {
        public virtual VideoPlayerList CreateVideoPlayers(int quantity)
        {
            if (quantity <= 0) return null;
            var players = new VideoPlayerList();
            for (var i = 0; i < quantity; i++)
            {
                players.Insert(new T());
            }
            return players;
        }
    }
    //两个具体批量创建工厂
    public class BatchRealPlayerFactory : BatchVideoPlayerFactory<RealPlayer> { }
    public class BatchMSMediaPlayerFactory : BatchVideoPlayerFactory<MSMediaPlayer> { }
}
