﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DesignPattern.FactoryMethodPattern.BatchFactory
{
    /// <summary>
    /// 生产指导者基类
    /// </summary>
    public abstract class Director
    {
        //步骤集合
        protected IList<Decision> decisions = new List<Decision>();
        //添加决策
        public virtual void Insert(Decision decision)
        {
            if (decision == null || decision.Factory == null || decision.Quantity <= 0)
            {
                return;
            }
            decisions.Add(decision);
        }
        //以便客户端遍历决策集合
        public virtual IEnumerable<Decision> Decisions
        {
            get { return decisions; }
        }
    }

    /// <summary>
    /// 视频播放器指导者
    /// </summary>
    public class VideoPlayerDirector : Director
    {
        public VideoPlayerDirector()
        {
            base.Insert(new RealPlayerDecision());
            base.Insert(new MSMediaPlayerDecision());
        }
        //对外加工的入口
        public VideoPlayerList Produce()
        {
            var players = new VideoPlayerList();
            foreach (var decision in Decisions)
            {
                players += decision.Factory.CreateVideoPlayers(decision.Quantity);
            }
            return players;
        }
    }
}
