﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DesignPattern.FactoryMethodPattern.FactoryMethod;

namespace DesignPattern.FactoryMethodPattern.BatchFactory
{
    /// <summary>
    /// 视频播放器集合类
    /// </summary>
    public class VideoPlayerList
    {
        private IList<VideoPlayer> players = new List<VideoPlayer>();

        #region 插入、删除、清除集合
        public void Insert(VideoPlayer item)
        {
            players.Add(item);
        }
        public void Insert(VideoPlayer[] items)
        {
            if (items == null || items.Length == 0) return;
            foreach (var item in items)
            {
                Insert(item);
            }
        }
        public void Remove(VideoPlayer item)
        {
            players.Remove(item);
        }
        public void Clear()
        {
            players.Clear();
        }
        #endregion

        //获取所有视频播放器的对象
        public VideoPlayer[] Items
        {
            get
            {
                if (players == null || players.Count == 0) return null;
                var items = new VideoPlayer[players.Count];
                players.CopyTo(items, 0);
                return items;
            }
        }
        public int Count
        {
            get { return players.Count; }
        }
        //为了便于操作,这里重载运算符+
        public static VideoPlayerList operator +(VideoPlayerList players, VideoPlayer[] items)
        {
            var result = new VideoPlayerList();
            if (players != null && players.Count > 0)
            {
                result.Insert(players.Items);
            }
            if (items != null && items.Length > 0)
            {
                result.Insert(items);
            }
            return result;
        }
        public static VideoPlayerList operator +(VideoPlayerList source, VideoPlayerList target)
        {
            var result = new VideoPlayerList();
            if (source != null && source.Count > 0)
            {
                result.Insert(source.Items);
            }
            if (target != null && target.Count > 0)
            {
                result.Insert(target.Items);
            }
            return result;
        }
    }
}
