﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using DesignPattern.FactoryMethodPattern.FactoryMethod;
using System.Configuration;
using System.Reflection;
using System.IO;
using DesignPattern.FactoryMethodPattern.BatchFactory;

namespace DesignPattern.UnitTest.Factory
{
    [TestClass]
    public class VideoPlayerTest
    {
        /*
        [TestMethod]
        public void TestRealPlayerFactory()
        {
            var realPlayer = RealPlayerFactory.CreateRealPlayer();
            Assert.IsNotNull(realPlayer);
            realPlayer.StartUp();//RealPlayer is starting...
            realPlayer.Play();//RealPlayer is playing...
            realPlayer.Pause(2);//RealPlayer is paused by 2 minutes.
            realPlayer.Stop();//RealPlayer has stopped.
        }

        [TestMethod]
        public void TestMSMediaPlayerFactory()
        {
            var msmediaPlayer = MSMediaPLayerFactory.CreateMSMediaPLayer();
            Assert.IsNotNull(msmediaPlayer);
            msmediaPlayer.StartUp();//MSMediaPLayer is starting...
            msmediaPlayer.Play();//MSMediaPLayer is playing...
            msmediaPlayer.Pause(3);//MSMediaPLayer is paused by 3 minutes.
            msmediaPlayer.Stop();//MSMediaPLayer has stopped.
        }
        */
        [TestMethod]
        public void TestVedioPlayerFactory()
        {
            var realPlayer = new RealPlayerFactory().CreateVideoPlayer();
            Assert.IsNotNull(realPlayer);
            realPlayer.StartUp();//RealPlayer is starting...
            realPlayer.Play();//RealPlayer is playing...
            realPlayer.Pause(2);//RealPlayer is paused by 2 minutes.
            realPlayer.Stop();//RealPlayer has stopped.

            var msmediaPlayer = new MSMediaPlayerFactory().CreateVideoPlayer();
            Assert.IsNotNull(msmediaPlayer);
            msmediaPlayer.StartUp();//MSMediaPLayer is starting...
            msmediaPlayer.Play();//MSMediaPLayer is playing...
            msmediaPlayer.Pause(3);//MSMediaPLayer is paused by 3 minutes.
            msmediaPlayer.Stop();//MSMediaPLayer has stopped.
        }

        [TestMethod]
        public void TestRealPlayerFactoryByConfig()
        {
            var factory = GetVedioPlayerFactoryByConfig("RealPlayerFormat");
            Assert.IsNotNull(factory);
            var player = factory.CreateVideoPlayer();
            Assert.IsNotNull(player);
            Assert.IsTrue(player.GetType() == typeof(RealPlayer));
        }

        [TestMethod]
        public void TestMSMediaPlayerFactoryByConfig()
        {
            var factory = GetVedioPlayerFactoryByConfig("MSMediaPlayerFormat");
            Assert.IsNotNull(factory);
            var player = factory.CreateVideoPlayer();
            Assert.IsNotNull(player);
            Assert.IsTrue(player.GetType() == typeof(MSMediaPlayer));
        }

        private VideoPlayerFactory GetVedioPlayerFactoryByConfig(string formatKey)
        {
            var format = ConfigurationManager.AppSettings[formatKey];
            var appPath = AppDomain.CurrentDomain.BaseDirectory;
            var assembly = Assembly.LoadFrom(Path.Combine(appPath, "DesignPattern.FactoryMethodPattern.dll"));
            //创建RealPlayerFactory
            var type = "DesignPattern.FactoryMethodPattern.FactoryMethod." + format + "Factory";
            var factory = (VideoPlayerFactory)assembly.CreateInstance(type);
            return factory;
        }

        [TestMethod]
        public void TestGenericRealPlayerFactory()
        {
            var format = ConfigurationManager.AppSettings["RealPlayerFormat"];
            var appPath = AppDomain.CurrentDomain.BaseDirectory;
            var assembly = Assembly.LoadFrom(Path.Combine(appPath, "DesignPattern.FactoryMethodPattern.dll"));
            //创建RealPlayerFactory
            var type = "DesignPattern.FactoryMethodPattern.FactoryMethod.Generic" + format + "Factory";
            var factory = (GenericVideoFactory<RealPlayer>)assembly.CreateInstance(type);
            Assert.IsNotNull(factory);
            var player = factory.CreateVideoPlayer();
            Assert.IsNotNull(player);
            Assert.IsTrue(player.GetType() == typeof(RealPlayer));
        }

        [TestMethod]
        public void TestGenericMSMediaPlayerFactory()
        {
            var format = ConfigurationManager.AppSettings["MSMediaPlayerFormat"];
            var appPath = AppDomain.CurrentDomain.BaseDirectory;
            var assembly = Assembly.LoadFrom(Path.Combine(appPath, "DesignPattern.FactoryMethodPattern.dll"));
            //创建RealPlayerFactory
            var type = "DesignPattern.FactoryMethodPattern.FactoryMethod.Generic" + format + "Factory";
            var factory = (GenericVideoFactory<MSMediaPlayer>)assembly.CreateInstance(type);
            Assert.IsNotNull(factory);
            var player = factory.CreateVideoPlayer();
            Assert.IsNotNull(player);
            Assert.IsTrue(player.GetType() == typeof(MSMediaPlayer));
        }

        [TestMethod]
        public void TestBatchVideoPlayerFactory()
        {
            var director = new VideoPlayerDirector();
            var players = director.Produce();//开始加工
            Assert.AreEqual<int>(3000 + 2000, players.Count);
            //验证前3000个播放器是RealPlayer
            for (var i = 0; i < 3000; i++) 
            {
                Assert.IsTrue(players.Items[i].GetType() == typeof(RealPlayer));
            }
            //验证后2000个播放器是MSMediaPlayer
            for (var i = 3000; i < 5000; i++)
            {
                Assert.IsTrue(players.Items[i].GetType() == typeof(MSMediaPlayer));
            }
        }
    }
}
